local ADDON, Package = ...
local Enum = Package.Enum
local Color = Package.Color
local ColorPalette = Package.ColorPalette

local agent = CreateFrame("Frame")
local registry = {}

local function load_storage()
  local schema = {
    version = 3,

    action_auto_transition = true,

    aura_duration = true,

    chat_sticky_raid_warning = true,
    chat_unclmap_frame = true,
    chat_max_history = true,
    chat_arrow_keys = true,
    chat_class_color = true,
    chat_message_logger = true,
    chat_copy_url_to_clipboard = true,
    chat_copy_text_to_clipboard = true,

    combat_disable_portrait_message = false,
    combat_disable_ui_error_message = false,

    cvar_camera_distance_max_zoom_factor = true,

    gametooltip_item_icon = true,
    gametooltip_item_stack_count = true,
    gametooltip_item_id = true,
    gametooltip_item_level = true,
    gametooltip_item_price = true,
    gametooltip_spell_icon = true,
    gametooltip_spell_id = true,
    gametooltip_aura_id = true,
    gametooltip_aura_icon = true,
    gametooltip_aura_caster = true,

    merchant_repair_items = true,
    merchant_sell_items = true,

    minimap_icon_drawer = true,
    minimap_icon_drawer_position = {Minimap:GetLeft(), Minimap:GetBottom()},
    minimap_zoom_in_or_out = true,
    minimap_world_map = true,

    paperdoll_item_quality_color_border = true,
    paperdoll_item_level = true,
    paperdoll_name = true,

    timealert_style = true,

    trade_recipient_name_with_class_and_level = true,
    trade_item_quality_color_border = true,
  }

  local schema_per_character = {
    version = 2,
    chat_log = {
      ChatFrame1 = {},
      ChatFrame3 = {},
      ChatFrame4 = {},
      ChatFrame5 = {},
      ChatFrame6 = {},
      ChatFrame7 = {},
      ChatFrame8 = {},
      ChatFrame9 = {},
      ChatFrame10 = {},
    }
  }

  MedusaPreludeStorage = Enum.merge(MedusaPreludeStorage or {}, schema, false)
  MedusaPreludeStoragePerCharacter = Enum.merge(MedusaPreludeStoragePerCharacter or {}, schema_per_character, false)
end

local function boot_up_messages()
  local addon = Color.format(ColorPalette.JORDY_BLUE, ADDON)
  local version = Color.format(ColorPalette.JORDY_BLUE, "v" .. GetAddOnMetadata(ADDON, "Version"))
  local slashes = Color.format(ColorPalette.SPRAY, _G["SLASH_" .. ADDON .. "1"] .. " " .. _G["SLASH_" .. ADDON .. "2"])

  DEFAULT_CHAT_FRAME:AddMessage(addon .. " " .. version .. " " .. slashes )
end

function registry.PLAYER_LOGIN(self)
  local function opt(key, is_detour, arg1, arg2, arg3, arg4, arg5)
    if MedusaPreludeStorage[key] or is_detour then
      Package[key](MedusaPreludeStorage[arg1], MedusaPreludeStorage[arg2], MedusaPreludeStorage[arg3], MedusaPreludeStorage[arg4], MedusaPreludeStorage[arg5])
    end
  end

  opt("action_auto_transition")

  opt("aura_duration")

  opt("chat_sticky_raid_warning")
  opt("chat_unclmap_frame")
  opt("chat_max_history")
  opt("chat_arrow_keys")
  opt("chat_class_color", true, "chat_class_color")
  opt("chat_message_logger")
  opt("chat_copy_text_to_clipboard")
  opt("chat_copy_url_to_clipboard")

  opt("combat_disable_portrait_message")
  opt("combat_disable_ui_error_message")

  opt("cvar_camera_distance_max_zoom_factor")

  opt("gametooltip_item", true, "gametooltip_item_icon", "gametooltip_item_stack_count", "gametooltip_item_id", "gametooltip_item_level", "gametooltip_item_price")
  opt("gametooltip_spell", true, "gametooltip_spell_icon", "gametooltip_spell_id")
  opt("gametooltip_aura", true, "gametooltip_aura_id", "gametooltip_aura_icon", "gametooltip_aura_caster")

  opt("merchant_repair_items")
  opt("merchant_sell_items")

  opt("minimap_icon_drawer", false, "minimap_icon_drawer_position")
  opt("minimap_zoom_in_or_out")
  opt("minimap_world_map")

  opt("paperdoll_item", true, "paperdoll_item_quality_color_border", "paperdoll_item_level")
  opt("paperdoll_name")

  opt("timealert_style")

  opt("trade_recipient_name_with_class_and_level")
  opt("trade_item_quality_color_border")
end

function registry.ADDON_LOADED(self, ...)
  if ADDON == ... then
    load_storage()
    boot_up_messages()
  end
end

agent:RegisterEvent("PLAYER_LOGIN")
agent:RegisterEvent("ADDON_LOADED")
agent:SetScript("OnEvent", function (self, event, ...) registry[event](self, ...) end)

SLASH_MedusaPrelude1 = "/" .. ADDON
SLASH_MedusaPrelude2 = "/MP"
SlashCmdList[ADDON] = function ()
  if not MedusaPreludeConfigWindow then
    Package.config()
  end

  MedusaPreludeConfigWindow:SetShown(not MedusaPreludeConfigWindow:IsShown())
end
