local ADDON, Package = ...
local Color = Package.Color
local ColorPalette = Package.ColorPalette

local function merchant_repair_items()
  local function OnEvent(self)
    if not IsShiftKeyDown() then
      local repair_all_cost, can_repair = GetRepairAllCost()
      local money = GetMoney()

      if can_repair and repair_all_cost <= money then
        RepairAllItems()

        DEFAULT_CHAT_FRAME:AddMessage(Color.format(ColorPalette.YELLOW, REPAIR_ALL_ITEMS) .. " " .. GetCoinTextureString(repair_all_cost):gsub(":0", ":-6"))
      end
    end
  end

  local agent = CreateFrame("Frame")
  agent:RegisterEvent("MERCHANT_SHOW")
  agent:SetScript("OnEvent", OnEvent)
end

local function merchant_sell_items()
  local function process()
    local items = 0
    local price = 0

    for bag_id = 0, NUM_BAG_SLOTS do
      for slot_id = 1, GetContainerNumSlots(bag_id) do
        local item_id = GetContainerItemID(bag_id, slot_id)

        if item_id then
          local _, item_count, item_locked, item_quality = GetContainerItemInfo(bag_id, slot_id)
          local _, _, _, _, _, _, _, _, _, _, item_sell_price = GetItemInfo(item_id)

          if not item_locked and item_quality == 0 and item_sell_price > 0 and MerchantFrame:IsShown() then
            items = items + 1
            price = price + (item_count * item_sell_price)

            C_Timer.After(0.01 + (items / 100), function () UseContainerItem(bag_id, slot_id) end)
          end
        end
      end
    end

    if items > 0 and price > 0 then
      DEFAULT_CHAT_FRAME:AddMessage(Color.format(ITEM_QUALITY_COLORS[0].color, ITEM_QUALITY0_DESC .. "x" .. items) .. " " .. GetCoinTextureString(price):gsub(":0", ":-6"))
    end
  end

  local function OnEvent(self)
    if not IsShiftKeyDown() then
      process()
    end
  end

  local agent = CreateFrame("Frame")
  agent:RegisterEvent("MERCHANT_SHOW")
  agent:SetScript("OnEvent", OnEvent)
end

Package.merchant_repair_items = merchant_repair_items
Package.merchant_sell_items = merchant_sell_items
